Package: dcmodify
Maintainer: Mark van der Loo <mark.vanderloo@gmail.com>
License: GPL-3
Title: Modify Data Using Externally Defined Modification Rules
Type: Package
LazyLoad: yes
Authors@R: c(person("Mark", "van der Loo"
                   , role   = c("cre","aut")
                   , email  = "mark.vanderloo@gmail.com"
                   , comment= c(ORCID="0000-0002-9807-4686"))
           , person("Edwin", "de Jonge"
                   , role = "aut"
                   , comment=c(ORCID="0000-0002-6580-4718"))
           , person("Sjabbo","Schaveling",  role = "ctb")
           , person("Floris", "Ruijter", role = "ctb") )
Description: Data cleaning scripts typically contain a lot of 'if this change that'
    type of statements. Such statements are typically condensed expert knowledge.
    With this package, such 'data modifying rules' are taken out of the code and
    become in stead parameters to the work flow. This allows one to maintain, document,
    and reason about data modification rules as separate entities.
Version: 0.9.0
Depends: methods
URL: https://github.com/data-cleaning/dcmodify
BugReports: https://github.com/data-cleaning/dcmodify/issues
Encoding: UTF-8
Imports: yaml, validate (>= 1.1.3), lumberjack (>= 1.3.1), settings,
        utils,
Suggests: simplermarkdown, tinytest,
VignetteBuilder: simplermarkdown
Collate: 'dplyr_verbs.R' 'guard.R' 'modifier.R' 'modify.R' 'validate.R'
RoxygenNote: 7.3.1
NeedsCompilation: no
Packaged: 2024-03-27 15:07:31 UTC; mark
Author: Mark van der Loo [cre, aut] (<https://orcid.org/0000-0002-9807-4686>),
  Edwin de Jonge [aut] (<https://orcid.org/0000-0002-6580-4718>),
  Sjabbo Schaveling [ctb],
  Floris Ruijter [ctb]
Repository: CRAN
Date/Publication: 2024-03-28 08:20:06 UTC
Built: R 4.6.0; ; 2026-04-26 00:21:29 UTC; unix
