# Generated by roxygen2: do not edit by hand

export("description<-")
export("label<-")
export("origin<-")
export(as_yaml)
export(description)
export(do_by)
export(export_yaml)
export(label)
export(max_by)
export(mean_by)
export(min_by)
export(modifier)
export(modify)
export(modify_so)
export(origin)
export(sum_by)
export(variables)
exportMethods(modify)
import(methods)
import(validate)
import(yaml)
importFrom(lumberjack,no_log)
importFrom(settings,clone_and_merge)
importFrom(utils,capture.output)
importFrom(validate,"description<-")
importFrom(validate,"label<-")
importFrom(validate,"origin<-")
importFrom(validate,as_yaml)
importFrom(validate,description)
importFrom(validate,do_by)
importFrom(validate,export_yaml)
importFrom(validate,label)
importFrom(validate,max_by)
importFrom(validate,mean_by)
importFrom(validate,min_by)
importFrom(validate,origin)
importFrom(validate,sum_by)
